import importlib
import os
import sys
import json
import time
from fastapi.staticfiles import StaticFiles

import databutton as db
from databutton.helpers import parse
from databutton.server.processes import start_processes
from fastapi import FastAPI
from starlette.responses import RedirectResponse

app = FastAPI()

sys.path.append(os.curdir)
imports = parse.find_databutton_directive_modules()
streamlit_modules = {}
for name in imports:
    streamlit_modules[name] = importlib.import_module(name)

# Write artifacts.json
artifacts = {
    'streamlit_apps': [{'route': k, 'filepath': v, 'last_updated': int(time.time() * 1000)} for k, v in db._streamlit_apps.items()]
}

with open('.databutton/artifacts.json', 'w') as f:
    json.dump(artifacts, f)

app.mount('/static', StaticFiles(directory='.databutton'), name='.databutton')

dir_path = os.path.dirname(os.path.realpath(__file__))
app_dir = os.path.join(dir_path, '../out')


@app.on_event('startup')
async def start_servers():
    print('Starting processes')
    await start_processes(app, db._streamlit_apps)

    @app.get('/')
    async def index():
        return RedirectResponse(url='/index.html')

    app.mount('/', StaticFiles(directory=app_dir), name='app')
